
//========================================================================//
// date : 2012/05/25
// brief : shader of skinning
// Created by Takashi Shibahara.
//========================================================================//
//- includes ---------------------------------------------------------------

//- define -----------------------------------------------------------------
#define MAX_BONE_MATRICES 128


//- constance number -------------------------------------------------------
#include	"Lighting.fxh"

cbuffer changeOnObj
{
	float4x4 boneTable[MAX_BONE_MATRICES];
}

texture	diffuseTex;

sampler diffuseTexSampler = sampler_state
{
    Texture = <diffuseTex>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//- structures difinition --------------------------------------------------
struct tagVS_INPUT
{
	float4 pos : POSITION;
    float4 color : COLOR;
	float3 normal : NORMAL;
	float2 diffuseUV : TEXCOORD0;

	float4 weights : BLENDWEIGHT;		// Bone weights
	int4 bones : BLENDINDICES;		// Bone indices

};

#define PER_PIXEL_LIGHTING	0

#if	PER_PIXEL_LIGHTING
struct tagPS_INPUT
{
    float4 pos : SV_POSITION;
    float4 color : COLOR;
	float2 diffuseUV : TEXCOORD0;
	float3 normal : TEXCOORD1;
};

#else

struct tagPS_INPUT
{
    float4 pos : SV_POSITION;
    float4 color : COLOR;
	float2 diffuseUV : TEXCOORD0;
};

#endif
//- functions --------------------------------------------------------------


//- vertex shader ----------------------------------------------------------
//--------------------------------------------------------------------------
// brief : ordinal vertex shader
// in 	 : 
// out   : 
// note  : nothing
//----------------------------------------------------	----------------------
tagPS_INPUT VS( tagVS_INPUT input )
{
	tagPS_INPUT output = (tagPS_INPUT)0;

	float4 skinnedPos = (float4)0;
	float3 skinnedNormal = (float3)0;

#if	1
	float4x4 bone = boneTable[ input.bones.x ];
	skinnedPos += input.weights.x *  mul( input.pos, bone );
	skinnedNormal += input.weights.x *  mul( input.normal, (float3x3)bone );

	bone = boneTable[input.bones.y];
	skinnedPos += input.weights.y *  mul( input.pos, bone );
	skinnedNormal += input.weights.y *  mul( input.normal, (float3x3)bone );

	bone = boneTable[input.bones.z];
	skinnedPos += input.weights.z *  mul( input.pos, bone );
	skinnedNormal += input.weights.z *  mul( input.normal, (float3x3)bone );

	bone = boneTable[input.bones.w];
	skinnedPos += input.weights.w *  mul( input.pos, bone );
	skinnedNormal += input.weights.w *  mul( input.normal, (float3x3)bone );
	
#else
	int4 indices = D3DCOLORtoUBYTE4(input.bones);
	float4x4 bone = boneTable[ indices[1] ];
	skinnedPos += input.weights.x *  mul( input.pos, bone );
	skinnedNormal += input.weights.x *  mul( input.normal, (float3x3)bone );

	bone = boneTable[indices[2]];
	skinnedPos += input.weights.y *  mul( input.pos, bone );
	skinnedNormal += input.weights.y *  mul( input.normal, (float3x3)bone );

	bone = boneTable[indices[3]];
	skinnedPos += input.weights.z *  mul( input.pos, bone );
	skinnedNormal += input.weights.z *  mul( input.normal, (float3x3)bone );

	bone = boneTable[indices[0]];
	skinnedPos += input.weights.w *  mul( input.pos, bone );
	skinnedNormal += input.weights.w *  mul( input.normal, (float3x3)bone );

#endif

	output.pos = mul( skinnedPos, wvp );

#if	PER_PIXEL_LIGHTING
	output.color = input.color;
#else
	float4 lightParam =
		CalculateLight( skinnedPos, skinnedNormal, input.color );

	output.color = lightParam * coloration;
#endif
	
	output.diffuseUV = input.diffuseUV;

	return output;
}

//- pixel shader -----------------------------------------------------------
//--------------------------------------------------------------------------
// brief : ordinal pixel shader
// in 	 : 
// out   : 
// note  : nothing
//--------------------------------------------------------------------------
float4 PS( tagPS_INPUT input ) : COLOR0
{
#if	PER_PIXEL_LIGHTING
	float4 lightParam =
		CalculateLight( input.pos, input.normal, input.color );

	float4	finalColor = tex2D( diffuseTexSampler, input.diffuseUV );
	finalColor *= lightParam * coloration;
#else
	float4	finalColor = input.color * tex2D( diffuseTexSampler, input.diffuseUV ); 

#endif
	return	finalColor;
}


// this is snipet for copy
//--------------------------------------------------------------------------
// brief : 
// in 	 : 
// out   : 
// note  : nothing
//--------------------------------------------------------------------------

//- technique --------------------------------------------------------------
technique Render
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_3_0, VS() ) );
        SetPixelShader( CompileShader( ps_3_0, PS() ) );
    }
}

